package algoritmy;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;
import konsts.Konsts.TypAkce;
import konsts.Konsts.VarAlgAJPOR;
import utility.DipUtils;

/**
 * Trida zajistuje provadeni algoritu Adaptivniho Jednopolynomialniho
 * Reseedingu (AJPOR) s vyuzitim fault masky. Celkovy beh algoritmu i simulace testu
 * se v porovnani s AutomatizaceAJPOR radove zrychli.
 *
 */
public class AutomatizaceAJPORMask {
    Properties properties = DipUtils.getProperties();
     
    private int ccaPocetIteraci = 0;
    int aktDetekPoruch = 0;
    private String aktChybovaMaska;
    private List<Integer> poruchy = new ArrayList<Integer>();
    private static String polynom;
    private String testVektorStart;
    private int poruchDetekMinIter;
    private int reseedPeek;
    private boolean staciJedna;   

    public void spustAJPORMask() throws Exception {       
        polynom = properties.getProperty("polynom");

        DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("generuj.testovaci.vektory.bat"));

        DipUtils.nactiVysledkyAtalanty(true);
        init(properties);
                    
        int prirustek = 0;
        String variantaAlg = properties.getProperty("varianta.algoritmu.ajporma");
        reseedPeek = Integer.valueOf(properties.getProperty("reseed.peek"));
        int stupen = Integer.valueOf(properties.getProperty("stupen.polynomu"));
        boolean generujSeed = properties.getProperty("generuj.seed").equalsIgnoreCase("1");
        boolean generujPolynom = properties.getProperty("generuj.polynom").equalsIgnoreCase("1");
        boolean pouzivatCA = properties.getProperty("pouzit.celularni.automat").equalsIgnoreCase("true");

        staciJedna = properties.getProperty("staci.jedna.porucha").equalsIgnoreCase("true");
        

        if (DipUtils.isSlozitejsiObvod()) {
            System.out.println(properties.getProperty("radek.zvyrazneni"));
            System.out.println("\nPOUZIVAS SLOZITEJSI OBVOD - uprav bataky i DipUtils.spustExternal !!!!\n");
            System.out.println(properties.getProperty("radek.zvyrazneni") + "\n");
        }

        try {            
            DipUtils.setPocetAktDetekovatPoruch("0");
            DipUtils.nastavPocDetekP = true;
            if (generujSeed) {
                testVektorStart = DipUtils.generujSeed(stupen);
            } else {
                testVektorStart = properties.getProperty("seed");
            }
            System.out.println("Seed:\n" + testVektorStart);

            if (generujPolynom) {
                polynom = DipUtils.generujPolynom(stupen);
            } else {
                polynom = properties.getProperty("polynom");
            }            
            System.out.println("Poly:\n" + polynom);

            if (pouzivatCA ){
                System.out.println("\nVektory pomoci CA!! \n");
            } else {
                System.out.println("\nVektory pomoci LZPR!! \n");
            }

            int krokuLZPR = Integer.valueOf(properties.getProperty("kroku.lzpr"));
                        
            //varianta dle prvniho schematu
            if (variantaAlg.equalsIgnoreCase(VarAlgAJPOR.TEST.toString())) {
                System.out.println("Spoustim variantu " + VarAlgAJPOR.TEST);
                //vygeneruju pocatecni sekvenci
                testVektorStart = sekvence(testVektorStart, krokuLZPR);
                aktDetekPoruch = getAktDetekPoruch();               
                prirustek = prirustekKrokLZPR(testVektorStart);
                
                while (!String.valueOf(aktDetekPoruch).equals(DipUtils.getMaxPocetDetekovatPoruch())) {
                    if ((int)prirustek < Integer.valueOf(properties.getProperty("procento.poruch"))) {                      
                        //bude reseding
                        prirustek = prirustekZReseedingu(krokuLZPR, -1);                        
                    } else {
                        //jinak dalsi LZPR krok, ...
                        prirustek = prirustekKrokLZPR(testVektorStart);                       
                    }
                }

            //varianta dle 2. schematu
            } else if (variantaAlg.equalsIgnoreCase(VarAlgAJPOR.PRETEST.toString())) {
                System.out.println("Spoustim variantu " + VarAlgAJPOR.PRETEST);
                prirustek = prirustekKrokLZPRPreTest(testVektorStart);
                while (!String.valueOf(aktDetekPoruch).equals(DipUtils.getMaxPocetDetekovatPoruch())) {
                    if (prirustek >= Integer.valueOf(properties.getProperty("procento.poruch"))) {
                        prirustek = prirustekKrokLZPRPreTest(getTestVektor());
                    } else {
                        prirustek =  prirustekReseedPreTest();
                    }
                }

            //varianta, kdy testujeme celou sekvenci
            } else if (variantaAlg.equalsIgnoreCase(VarAlgAJPOR.SEKV.toString())) {
                System.out.println("Spoustim variantu " + VarAlgAJPOR.SEKV);
                System.out.println("");
                prirustek = prirustekSekvPreTest(testVektorStart, krokuLZPR);
                setTestVektor(testVektorStart);
                prirustekSekvPreTestVoid(getTestVektor(), krokuLZPR);

            } else {
                throw new Exception(properties.getProperty("varianta.alg.error"));
            }

        } catch (Exception ex) {
            ex.printStackTrace();      
       }
    }   
    
    /**
     * Metoda zapise prvni vektor do testu, provede krok LZPR, 
     * vyhodnoti, zda prirustek detekce poruch novym vektorem je dostatecny a 
     * pokud ano, vektor je pripsan do testu, pokud ne, nastane reseeding.
     *
     * @param testVektor
     * @return
     * @throws Exception
     */
    private int prirustekKrokLZPRPreTest(String testVektor) throws Exception {
        //System.out.println("Krok LZPR pre test...");
              
        int refPocDetekPoruch = 0;
        int aktDetPoruch = 0;
        int pomDetPoruch = 0;
        int prirustek = 0;
        double procIter = Double.parseDouble(properties.getProperty("procento.iteraci"));       
        String chybMaskaZal = getAktChybovaMaska();
       
        if (getPoruchy().size() == 0) {
            refPocDetekPoruch = 0;
        } else {
            refPocDetekPoruch = getPoruchy().get(DipUtils.zaokrouhli
                    (ccaPocetIteraci - (ccaPocetIteraci * (procIter/100))));
        }
        
        //zapis vektor do testovaci sady (vysledky s timto vektorem me zajimaji vzdy!!!)
        krokPreTest(testVektor, true);
        //a ziskej aktualni chybovou masku jako vysledek        
        chybMaskaZal = DipUtils.getVysledekOperaceOR(
                DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska")), chybMaskaZal);        
        pomDetPoruch = aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);
        getPoruchy().add(aktDetPoruch);        
        //tento pocet detekovanych poruch vzdy zapis
        DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch);        
        
        // toto nastav vzdy
        setPoruchDetekMinIter(aktDetPoruch);
        // a vzdy musis zvysit pocet iteraci, kvuli spravnemu indexovani do poruch
        setCcaPocetIteraci(getCcaPocetIteraci() + 1);
        //a zapamatuj si (nastav globalne) aktualni chybovou masku
        setAktChybovaMaska(chybMaskaZal);
                
        //proved krok LFSR
        testVektor = DipUtils.krokLZPR(polynom, testVektor);
        //a zapamatuj si dany vektor
        setTestVektor(testVektor);
        //otestuj ho, zda splnuje podminku pro zarazeni do testovaci sady - dle prirustku detekce poruch
        krokPreTest(testVektor, false);
       
        chybMaskaZal = DipUtils.getVysledekOperaceOR(
                DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska")), chybMaskaZal);
        aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);       

        //splnuje podminku? if ANO -> do testu if NE -> reseeding                
        prirustek = DipUtils.spoctiPrirustek(aktDetPoruch, refPocDetekPoruch);        

        if (getAktDetekPoruch() != 0) {
            logujPrubeh(TypAkce.LZPR, getAktDetekPoruch(), pomDetPoruch);
        }
        
        if (prirustek >= Integer.valueOf(properties.getProperty("procento.poruch"))) {
            //zapis novy vektor do testu
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), testVektor, true);
            DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch);            
            setAktChybovaMaska(chybMaskaZal);
            setPoruchDetekMinIter(aktDetPoruch);
            setAktDetekPoruch(aktDetPoruch);
            getPoruchy().add(aktDetPoruch);
            setCcaPocetIteraci(getCcaPocetIteraci() + 1);
            //a proved dalsi krok LZPR a nastav globalne tento vektor jako testovaci
            setTestVektor(DipUtils.krokLZPR(polynom, testVektor));
            
             logujPrubeh(TypAkce.LZPR, pomDetPoruch, aktDetPoruch);
        } else {
            logujPrubeh(TypAkce.Reseeding, pomDetPoruch, aktDetPoruch);
        }

        return prirustek;
    }


    /**
     * Metoda zapise prvni vektor do testu. Nasledne provadi sekvenci LZPR kroku,
     * a otestuje zda prirustek v ramci cele sekvence byl dostatecny. Pokud ano,
     * vektory sekvence jsou zapsany do testu a poruchy zapsany do souboru,
     * pokud ne, nastane reseeding.
     *
     * @param testVektor
     * @param polynom
     * @param delkaSekv
     * @return
     * @throws Exception
     */
    private int prirustekSekvPreTest(String testVektor, int delkaSekv) throws Exception {
        int refPocDetekPoruch = 0;
        int aktDetPoruch = 0;        
        int prirustek = 0;
        int procPoruch = Integer.valueOf(properties.getProperty("procento.poruch"));
        String chybMaskaZal = getAktChybovaMaska();
        List<String> seznamVektoru = new ArrayList<String>();
        Integer[] detekPoruchy = new Integer[delkaSekv];

        boolean pouzitCA = properties.getProperty("pouzit.celularni.automat").equalsIgnoreCase("true");

        seznamVektoru.clear();        
        
        //zapis vektor do testovaci sady (vysledky s timto vektorem me zajimaji vzdy!!!)
        krokPreTest(testVektor, true);
        //a ziskej aktualni chybovou masku jako vysledek        
        chybMaskaZal = DipUtils.getVysledekOperaceOR(
                DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska")), chybMaskaZal);
        aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);        
        getPoruchy().add(aktDetPoruch);
        //System.out.println("Detekce krok LZPR: " + aktDetPoruch);
        //tento pocet detekovanych poruch vzdy zapis
        DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch);
        logujProExcel(aktDetPoruch, 0, 0);

        // toto nastav vzdy
        setPoruchDetekMinIter(aktDetPoruch);
        // a vzdy musis zvysit pocet iteraci, kvuli spravnemu indexovani do poruch
        setCcaPocetIteraci(getCcaPocetIteraci() + 1);
        //a zapamatuj si (nastav globalne) aktualni chybovou masku
        setAktChybovaMaska(chybMaskaZal);
        refPocDetekPoruch = aktDetPoruch;

        for (int i = 0; i < delkaSekv; i++) {
            //proved krok LZPR
            testVektor = DipUtils.krokLZPR(polynom, testVektor);
            if (pouzitCA) {
                testVektor = DipUtils.getVektorPravidlo60(testVektor);
            }
            seznamVektoru.add(testVektor);
            //a zapamatuj si dany vektor
            setTestVektor(testVektor);
            //otestuj ho, zda splnuje podminku pro zarazeni do testovaci sady - dle prirustku detekce poruch
            krokPreTest(testVektor, false);          
            chybMaskaZal = DipUtils.getVysledekOperaceOR(
                    DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska")), chybMaskaZal);
            aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);
            detekPoruchy[i] = aktDetPoruch;
            System.out.println(aktDetPoruch);
        }       
        
        //splnuje podminku? if ANO -> do testu if NE -> reseeding
        if (staciJedna) {
            prirustek = (aktDetPoruch-refPocDetekPoruch) >=1 ? 10 : 0;
        } else {
            prirustek = DipUtils.spoctiPrirustek(aktDetPoruch, refPocDetekPoruch);
        }

        if (prirustek >= procPoruch) {
            //zapis sekvenci vektoru do testu
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), seznamVektoru, true);
            for (int i = 0; i < detekPoruchy.length; i++) {
                DipUtils.zapisPocetDetekDoSouboru(detekPoruchy[i]);
                logujProExcel(detekPoruchy[i], 0, 0);
            }
            setAktChybovaMaska(chybMaskaZal);
            setPoruchDetekMinIter(aktDetPoruch);
            setAktDetekPoruch(aktDetPoruch);
            getPoruchy().addAll(Arrays.asList(detekPoruchy));
            setCcaPocetIteraci(getCcaPocetIteraci() + 1);
            //a proved dalsi krok LZPR a nastav globalne tento vektor jako testovaci
            setTestVektor(DipUtils.krokLZPR(polynom, testVektor));            
            //logujPrubeh(TypAkce.LZPR, refPocDetekPoruch, aktDetPoruch);
            //logujProExcel(aktDetPoruch, 0, 0);            
        } else {
            //logujPrubeh(TypAkce.Reseeding, refPocDetekPoruch, aktDetPoruch);
            logujProExcel(aktDetPoruch, 1, reseedPeek);
            setTestVektor(DipUtils.getNovySeed());
        }       
        return prirustek;
    }

    private void prirustekSekvPreTestVoid(String testVektor, int delkaSekv) throws Exception {       
        int refPocDetekPoruch = 0;
        int aktDetPoruch = 0;
        int prirustek = 0;
        int procPoruch = Integer.valueOf(properties.getProperty("procento.poruch"));
        List<String> seznamVektoru = new ArrayList<String>();
        String chybMaskaZal = getAktChybovaMaska();
        Integer[] detekPoruchy = new Integer[delkaSekv];
        boolean pouzitCA = properties.getProperty("pouzit.celularni.automat").equalsIgnoreCase("true");

        while(true) {           
            seznamVektoru.clear();
            //zapis vektor do testovaci sady (vysledky s timto vektorem me zajimaji vzdy!!!)
            krokPreTest(testVektor, true);
            //a ziskej aktualni chybovou masku jako vysledek
            chybMaskaZal = DipUtils.getVysledekOperaceOR(
                    DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska")), chybMaskaZal);
            aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);
            getPoruchy().add(aktDetPoruch);
            //tento pocet detekovanych poruch vzdy zapis
            DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch);
            logujProExcel(aktDetPoruch, 0, 0);
            
            refPocDetekPoruch = aktDetPoruch;            
            
            for (int i = 0; i < delkaSekv; i++) {
                //proved krok LZPR
                testVektor = DipUtils.krokLZPR(polynom, testVektor);
                if (pouzitCA) {
                    testVektor = DipUtils.getVektorPravidlo60(testVektor);
                }
                seznamVektoru.add(testVektor);               
                //otestuj ho, zda splnuje podminku pro zarazeni do testovaci sady - dle prirustku detekce poruch
                krokPreTest(testVektor, false);
                chybMaskaZal = DipUtils.getVysledekOperaceOR(
                        DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska")), chybMaskaZal);
                aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);
                detekPoruchy[i] = aktDetPoruch;
                //System.out.println(aktDetPoruch);
            }
            //a zapamatuj si dany posledni vektor
            setTestVektor(testVektor);

            //splnuje podminku? if ANO -> do testu if NE -> reseeding
            prirustek = (aktDetPoruch-refPocDetekPoruch) >=1 ? 10 : 0;
            
            if (prirustek >= procPoruch) {
                //zapis sekvenci vektoru do testu
                DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), seznamVektoru, true);
                for (int i = 0; i < detekPoruchy.length; i++) {
                    DipUtils.zapisPocetDetekDoSouboru(detekPoruchy[i]);
                    logujProExcel(detekPoruchy[i], 0, 0);
                }                             
                getPoruchy().addAll(Arrays.asList(detekPoruchy));
                //a proved dalsi krok LZPR a nastav globalne tento vektor jako testovaci
                setTestVektor(DipUtils.krokLZPR(polynom, testVektor));                
                //logujProExcel(aktDetPoruch, 0, 0);
            } else {                
                logujProExcel(aktDetPoruch, 1, reseedPeek);
                testVektor = DipUtils.getNovySeed();
                setTestVektor(testVektor);
            }
        }
    }

    /**
     *  TODO
     * 
     * @param properties
     * @return
     * @throws Exception
     */
    private int prirustekReseedPreTest() throws Exception {       
        int aktDetPoruch;
        int pomDetPoruch;
        int prirustek;
        int procPoruch = Integer.valueOf(properties.getProperty("procento.poruch"));
        int refPocDetekPoruch = getPoruchDetekMinIter();
        String chybMaskaZal = getAktChybovaMaska();
        
        String testVektor = DipUtils.getNovySeed();

        getPoruchy().clear();
        setCcaPocetIteraci(0);
        
        //zapis novy vektor do testu
        krokPreTest(testVektor, true);
        //a ziskej aktualni chybovou masku jako vysledek        
        chybMaskaZal = DipUtils.getVysledekOperaceOR(
                DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska")), chybMaskaZal);
        aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);
        pomDetPoruch = aktDetPoruch;
        getPoruchy().add(aktDetPoruch);
        //tento pocet detekovanych poruch vzdy zapis
        DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch);
        logujProExcel(aktDetPoruch, 1, reseedPeek);

        // toto nastav vzdy
        setPoruchDetekMinIter(aktDetPoruch);
        //a zapamatuj si (nastav globalne) aktualni chybovou masku
        setAktChybovaMaska(chybMaskaZal);               

        //proved krok LZPR
        testVektor = DipUtils.krokLZPR(polynom, testVektor);
        //a zapamatuj si dany vektor
        setTestVektor(testVektor);
        //otestuj ho, zda splnuje podminku pro zarazeni do testovaci sady - dle prirustku detekce poruch
        krokPreTest(testVektor, false);       
        chybMaskaZal = DipUtils.getVysledekOperaceOR(
                DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska")), chybMaskaZal);
        aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);        

        //splnuje podminku? if ANO -> do testu if NE -> reseeding
        if (staciJedna) {
            prirustek = (aktDetPoruch-refPocDetekPoruch) >=1 ? procPoruch : 0;
        } else {
            prirustek = DipUtils.spoctiPrirustek(aktDetPoruch, refPocDetekPoruch);
        }

        if (prirustek >= procPoruch) {
            //zapis novy vektor do testu
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), testVektor, true);
            DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch);           
            setAktChybovaMaska(chybMaskaZal);
            setPoruchDetekMinIter(aktDetPoruch);
            setAktDetekPoruch(aktDetPoruch);
            getPoruchy().add(aktDetPoruch);
            setCcaPocetIteraci(getCcaPocetIteraci() + 1);
            //a proved dalsi krok LZPR a nastav globalne tento vektor jako testovaci
            setTestVektor(DipUtils.krokLZPR(polynom, testVektor));

            logujPrubeh(TypAkce.LZPR, pomDetPoruch, aktDetPoruch);
            logujProExcel(aktDetPoruch, 0, 0);
        } else {
            logujPrubeh(TypAkce.Reseeding, pomDetPoruch, aktDetPoruch);
            //logujProExcel(aktDetPoruch, 1, reseedPeek);
        }

        return prirustek;
    }

    /**
     * Metoda otestuje prirustek detekce poruch z daneho vektoru.
     *
     * @param testVektor
     * @param zapisDoTestu
     * @throws Exception
     */
    private void krokPreTest(String testVektor, boolean zapisDoTestu) throws Exception {
        if(zapisDoTestu) {
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), testVektor, true);
        }
        DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.mask.pat"), testVektor, false);
        if(!DipUtils.isSlozitejsiObvod()) {
            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("lzprMask.soubor.bat"));
        } else {
            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("atalanta.lzpr.mask"));
        }
    }

    /**
     * Metoda provede sekvenci kroku LZPR delky pocetKrokuLZPR, pocet detekovanych
     * poruch v jednotlivych krocich zapisuje do souboru, testovaci vektory tez zapisuje
     * do souboru. Vystupem je jeden posledni vektr dane sekvence.
     *   
     * @param testVektor
     * @param pocetKrokuLZPR   
     * @return testVektor
     * @throws Exception
     */
    private String sekvence(String testVektor, int pocetKrokuLZPR) throws Exception {       
        String chybMaskaZal = getAktChybovaMaska();
        List<String> vektory = new ArrayList<String>();
        int detekPoruch = 0;
        List<Integer> detekPoruchy = new ArrayList<Integer>();       
        for (int i = 0; i < pocetKrokuLZPR; i++) {
            vektory.add(testVektor);
            
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.mask.pat"), testVektor, false);            
            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("lzprMask.soubor.bat"));
                        
            chybMaskaZal = DipUtils.getVysledekOperaceOR(
                    DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska")), chybMaskaZal);
            detekPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);            
            
            detekPoruchy.add(detekPoruch);
            DipUtils.zapisPocetDetekDoSouboru(detekPoruch);
            testVektor = DipUtils.krokLZPR(polynom, testVektor);
        }
        DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), vektory, true);        
        setAktDetekPoruch(DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal));
        setCcaPocetIteraci(pocetKrokuLZPR);
        setAktChybovaMaska(chybMaskaZal);
        DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.chybova.maska"), chybMaskaZal, false);
        pridejZaznamy(detekPoruchy);
        System.out.println("--------zacinam merit----------");

        return testVektor;
    }

    /**
     * Metoda spocte a vrati prirustek za urcite procento iteraci.
     *
     * @param testVektor
     * @return prirustek za procento iteraci
     * @throws Exception
     */
    private int prirustekKrokLZPR(String testVektor) throws Exception {
        //System.out.println("Krok LZPR...");

        int refIndex;
        int refPocDetekPoruch;
        double procIter = Double.parseDouble(properties.getProperty("procento.iteraci"));

        //TODO
        refIndex = DipUtils.zaokrouhli(ccaPocetIteraci - (ccaPocetIteraci * (procIter/100)));
        refPocDetekPoruch = getPoruchy().get(refIndex);
        
        //System.out.println("cca iteraci: " + getCcaPocetIteraci() + " ref. pocet poruch:" + refPocDetekPoruch);
        int aktDetPoruch = 0;
        int prirustek = 0;
        
        String chybMaska = "";
        String chybMaskaZal = getAktChybovaMaska();
        
        DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.mask.pat"), testVektor, false);
        DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), testVektor, true);
        DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
            properties.getProperty("lzprMask.soubor.bat"));

        chybMaska = DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska"));
        chybMaskaZal = DipUtils.getVysledekOperaceOR(chybMaska, chybMaskaZal);
        aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);
        DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch);
        
        setAktChybovaMaska(chybMaskaZal);
        testVektor = DipUtils.krokLZPR(polynom, testVektor);
        setTestVektor(testVektor);
        setAktDetekPoruch(aktDetPoruch);
        getPoruchy().add(aktDetPoruch);
        setCcaPocetIteraci(getCcaPocetIteraci() + 1);

        if (getPoruchDetekMinIter() == 0) {
            //System.out.println("Pred reseedingem ref." + refPocDetekPoruch);
            prirustek = DipUtils.spoctiPrirustek(aktDetPoruch, refPocDetekPoruch);
            //info(refPocDetekPoruch, prirustek);
            logujPrubeh(TypAkce.LZPR, refPocDetekPoruch, aktDetPoruch);
        } else {
            //System.out.println("V reseedingu ref.: " + refPocDetekPoruch);
            prirustek = DipUtils.spoctiPrirustek(aktDetPoruch - getPoruchDetekMinIter(),
                    refPocDetekPoruch - getPoruchDetekMinIter());
            //info(refPocDetekPoruch, prirustek);
            logujPrubeh(TypAkce.Reseeding, refPocDetekPoruch, aktDetPoruch);
        }

        return prirustek;
    }

    
      
    /**
     * Metoda vytvori sekvenci reseedingu a vrati prirustek detekce poruch z 
     * dane sekvence, tedy rozdil poctu detekovanych poruch mezi zacatkem a
     * koncem sekvence.
     *
     * @param refIndex index do pole poruch, pokud ruzny od -1, setuji index ref. poruchy
     * @return prirustek
     */
    private int prirustekZReseedingu(int krokuLZPR, int refIndex) throws Exception {        
        setCcaPocetIteraci(0);
        String testVektorRes = "";
        int prirustek = 0;
        int refPocDetekPoruch = 0;
        int aktDetekPoruchRes = 0;
        double procIter = Double.parseDouble(properties.getProperty("procento.iteraci"));

        //uchovam si pocet detekovanych poruch v predchozim behu
        setPoruchDetekMinIter(getPoruchy().get(poruchy.size()-1));
        getPoruchy().clear();
        System.out.println("##########Reseeding##################");       

        testVektorRes = DipUtils.getNovySeed();
        testVektorRes = sekvence(testVektorRes, krokuLZPR);
        //posledni vektor si musim pamatovat
        setTestVektor(testVektorRes);
        // sekvence pripsala do poruchy dalsi data a zajima me posledni zapsana hodnota

        aktDetekPoruchRes = getPoruchy().get(poruchy.size() - 1);
        refIndex = DipUtils.zaokrouhli(ccaPocetIteraci - (ccaPocetIteraci * (procIter/100)));
        refPocDetekPoruch = getPoruchy().get(refIndex);
        
        prirustek = DipUtils.spoctiPrirustek(aktDetekPoruchRes-getPoruchDetekMinIter(),
                refPocDetekPoruch-getPoruchDetekMinIter());
        System.out.println("Prirustek v reseedingu:" + prirustek);
       
        //info(refPocDetekPoruch, prirustek);
        logujPrubeh(TypAkce.Reseeding, refPocDetekPoruch, aktDetekPoruchRes );
        return prirustek;
    }
    

    /**
     * Metoda provadi krok LZPR dokud se kazdym dalsim krokem detekuje alespon o
     * jednu poruchu vice. Pokud se tak nestane, provede se reseeding.
     *
     * @param testVektor
     * @throws Exception
     */
    private void detekovanyPrirustek(String testVektor) throws Exception {
        String chybMaska;
        String chybMaskaZal = getAktChybovaMaska();
        int initDetPoruch = 0;
        int aktDetPoruch;
       
        /*
         *zapis vektor do testu, nebot zcela jiste dodetekoval dalsi poruchy,
         * protoze tato metoda je volana po reseedingu, nebo po kroku LZPR
         */
        krokPreTest(testVektor, true);        
        //a ziskej aktualni chybovou masku jako vysledek
        chybMaska = DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska"));
        chybMaskaZal = DipUtils.getVysledekOperaceOR(chybMaska, chybMaskaZal);
        //zjisti, kolik detekoval poruch
        aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);
        initDetPoruch = aktDetPoruch;
        
        //tento pocet detekovanych poruch vzdy zapis
        DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch);      
        
        //a zapamatuj si (nastav globalne) aktualni chybovou masku
        setAktChybovaMaska(chybMaskaZal);        

        //proved krok LZPR
        testVektor = DipUtils.krokLZPR(polynom, testVektor);      
        //delej kroky LZPR tak dlouho, dokud je aspon nejaky prirustek
        while(!String.valueOf(getAktDetekPoruch()).equals(DipUtils.getMaxPocetDetekovatPoruch())) {
            
            krokPreTest(testVektor, false);           
            chybMaskaZal = DipUtils.getVysledekOperaceOR(
                    DipUtils.nactiRadekDat(properties.getProperty("soubor.chybova.maska")), chybMaskaZal);
            aktDetPoruch = DipUtils.getPocetDetekPoruchZMasky(chybMaskaZal);

            if (aktDetPoruch != initDetPoruch) {
                //ano, byl prirustek
                DipUtils.zapisPocetDetekDoSouboru(aktDetPoruch);
                setAktChybovaMaska(chybMaskaZal);               
                DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), testVektor, true);
                logujPrubeh(TypAkce.LZPR, initDetPoruch, aktDetPoruch);
                initDetPoruch = aktDetPoruch;
                setAktDetekPoruch(aktDetPoruch);
                //proved krok LZPR
                testVektor = DipUtils.krokLZPR(polynom, testVektor);                
            } else {
                logujPrubeh(TypAkce.Reseeding, initDetPoruch, aktDetPoruch);
                testVektor = DipUtils.getNovySeed();
            }            
        }
        //System.out.println(aktDetPoruch);
    }    

     /**
     * Metoda vyprazdni vsechny potrebne soubory.
     *
     * @param properties
     * @throws Exception
     */
    private void init(Properties properties) throws Exception {
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.atalanta.rep"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.atalanta.pat"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.detekovane.poruchy"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.poruchy.zaloha"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.poruchy.prirustky"));
    }
    
    /**
     * Metoda prehledne zaloguje do souboru prubeh algritmu.
     *
     * @param typAkce
     * @param initDetPoruch
     * @param aktDetPoruch
     * 
     * @throws Exception
     */
    private void logujPrubeh(TypAkce typAkce, int initDetPoruch, int aktDetPoruch) throws Exception {
        String typAkceS = typAkce.toString();
        if(typAkceS.equalsIgnoreCase(TypAkce.LZPR.toString())) {
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.poruchy.prirustky"),
                String.valueOf(initDetPoruch) + "\t" + String.valueOf(aktDetPoruch) + "\t V2 -> test, LFSR", true);
        } else {
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.poruchy.prirustky"),
                String.valueOf(initDetPoruch) + "\t" + String.valueOf(aktDetPoruch) + "\t reseeding ", true);
        }
    }

    /**
     *
     * @param initDetPoruch
     * @param typAkce 0-krok LZPR, 1-reseeding
     * @param hodnota
     * @throws Exception
     */
    private void logujProExcel(int initDetPoruch, int typAkce, int hodnota) throws Exception {
        if (typAkce == 0) {
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.poruchy.zaloha"),
                    String.valueOf(initDetPoruch) + ";" + "0", true);
        } else if (typAkce == 1) {
            DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.poruchy.zaloha"),
                    String.valueOf(initDetPoruch) + ";" + hodnota, true);
        }
    }

    
    /**
     * Metoda vypisuje info o prubehu algoritmu.
     *
     * @param refPocDetekPoruch
     * @param prirustek
     */
    private void info(double refPocDetekPoruch, double prirustek) {      
        System.out.println("Akt. detek:   " + getAktDetekPoruch());
        System.out.println("Ref. detek:   " + (int)refPocDetekPoruch);
        System.out.println("Det. min. it: " + getPoruchDetekMinIter());            
        System.out.println("Prirustek: " + prirustek + "% za " + Integer.valueOf(properties.getProperty("procento.iteraci"))
            +"% iteraci, kdyz rozdil mensi nez " + Integer.valueOf(properties.getProperty("procento.poruch")) + "%, pak reseeding.");
    }

    
    
    // getters and setters


    public int getCcaPocetIteraci() {
        return ccaPocetIteraci;
    }

    public void setCcaPocetIteraci(int ccaPocetIteraci) {
        this.ccaPocetIteraci = ccaPocetIteraci;
    }

    public String getAktChybovaMaska() {
        return aktChybovaMaska;
    }

    public void setAktChybovaMaska(String aktChybovaMaska) {
        this.aktChybovaMaska = aktChybovaMaska;
    }

    public int getAktDetekPoruch() {
        return aktDetekPoruch;
    }

    public void setAktDetekPoruch(int aktDetekPoruch) {
        this.aktDetekPoruch = aktDetekPoruch;
    }

    public List<Integer> getPoruchy() {
        return poruchy;
    }

    public void setPoruchy(List<Integer> poruchy) {
        this.poruchy = poruchy;
    }

    private void pridejZaznamy(List<Integer> poruchy) {
        getPoruchy().addAll(poruchy);
    }

    public String getTestVektor() {
        return testVektorStart;
    }

    public void setTestVektor(String testVektor) {
        this.testVektorStart = testVektor;
    }

    public int getPoruchDetekMinIter() {
        return poruchDetekMinIter;
    }

    public void setPoruchDetekMinIter(int poruchDetekMinIter) {
        this.poruchDetekMinIter = poruchDetekMinIter;
    }
  
}