package main;

import algoritmy.AlgLinZpetVazReg;
import algoritmy.AutomatizaceAJPOR;
import algoritmy.AutomatizaceAJPORMask;
import entity.AlgoritmyVycet;
import entity.RadekDatDavka;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import utility.DipUtils;

/**
 * Main trida spoustejici cely program bez GUI. Algoritmus je vybran dle parametru z
 * prikazoveho radku. Pokud nastane vyjimka, progra se ukonci a vypise chybovou
 * zpravu.
 *
 */
public class SpustBezGUI {

    static Properties properties;

    /**
     * Vlastni metoda main. Spousti cely program.
     *
     * @param args - ocekavan pouze jeden parametr - oznaceni algoritmu
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
        properties = DipUtils.nactiProperties();

        if (args == null || args.length != 1) {
            throw new Exception(properties.getProperty("param.error"));
        }

        spustDleParametru(args[0]);

    }

    /**
     * Metoda dle parametru spusti prislusny algoritmus.
     *
     * @param oznacAlg - oznaceni algoritmu
     * @see AlgoritmyVycet
     * @throws Exception
     */
    private static void spustDleParametru(String oznacAlg) throws Exception {
        if (oznacAlg == null) {
            throw new Exception(properties.getProperty("alg.type.error"));
        }

        if (oznacAlg.equalsIgnoreCase(AlgoritmyVycet.LZPR.name())) {            
            List<RadekDatDavka> dataDavky = DipUtils.nactiDataZeSouboru(properties.getProperty("soubor.AJPORvsLZPR.davka"));

            int j = 0;
            for (RadekDatDavka radekDavky : dataDavky) {
                init(properties);
                properties.setProperty("soubor.detekovane.poruchy",
                    properties.getProperty("soubor.detekovane.poruchy.zal"));
                properties.setProperty("soubor.detekovane.poruchy",
                    properties.getProperty("soubor.detekovane.poruchy.zal") + "_" + j);

                String testVektor = radekDavky.getSeed();
                String polynom = radekDavky.getPolynom();
                System.out.println(radekDavky);

                for (int i = 0; i < radekDavky.getPocetVektoru(); i++) {
                    DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), testVektor, true);
                    DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                            properties.getProperty("lzpr.soubor.bat"));
                    DipUtils.nactiVysledkyAtalanty();                    
                    DipUtils.zapisPocetDetekDoSouboru(DipUtils.getAktPocetDetekPoruch());
                    testVektor = DipUtils.krokLZPR(polynom, testVektor);
                }
                Thread.sleep(500);
                j++;
            }
        }

        if (oznacAlg.equalsIgnoreCase(AlgoritmyVycet.LZPRAJPOR.name())) {
            AlgLinZpetVazReg alzvr = new AlgLinZpetVazReg();
            List<String> lzprSekv = new ArrayList<String>();
            List<RadekDatDavka> dataDavky = DipUtils.nactiDataZeSouboru(properties.getProperty("soubor.AJPORvsLZPR.davka"));
            for (RadekDatDavka radekDavky : dataDavky) {
                lzprSekv = alzvr.vytvorSekvenci(radekDavky.getPolynom(), radekDavky.getSeed(), 10 * radekDavky.getPocetVektoru());
                DipUtils.zapisVektoryDoSouboru(properties.getProperty("soubor.atalanta.pat"), lzprSekv, false);
                DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                        properties.getProperty("lzpr.soubor.bat"));
                DipUtils.nactiVysledkyAtalanty();                
                DipUtils.zapisPocetDetekDoSouboru(DipUtils.getAktPocetDetekPoruch());
            }

        } else if (oznacAlg.equalsIgnoreCase(AlgoritmyVycet.JPOR.name())) {
            //TODO
        } else if (oznacAlg.equalsIgnoreCase(AlgoritmyVycet.AJPOR.name())) {
            AutomatizaceAJPOR aJPOR = new AutomatizaceAJPOR();
            aJPOR.spustAJPOR();
        } else if (oznacAlg.equalsIgnoreCase(AlgoritmyVycet.AJPORMA.name())) {
            AutomatizaceAJPORMask aJPORMask = new AutomatizaceAJPORMask();
            aJPORMask.spustAJPORMask();
        } else {
            throw new Exception(properties.getProperty("alg.type.error"));
        }
    }

    private static void init(Properties properties) throws Exception {
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.atalanta.rep"));
        DipUtils.vyprazdniSoubor(properties.getProperty("soubor.atalanta.pat"));
        //DipUtils.vyprazdniSoubor(properties.getProperty("soubor.detekovane.poruchy"));
    }
}
