package utility;

import entity.RadekDat;
import entity.RadekDatDavka;
import entity.VektorAFitness;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.Random;

/**
 * Trida poskytuje staticke metody pouzite napric celym projektem.
 *
 */
public class DipUtils {

    static Properties properties;   
    private static String pocetDetekovatPoruch = "";
    private static String aktPocetDetekPoruch = "";
    private static final String D_FAULTS_STRING = "d_faults:";
    public static boolean nastavPocDetekP = false;   
    public static boolean slozitejsiObvod;

    /**
     * Metoda vrati property soubor se vsemi klici a hodnotami.
     *
     * @return properties
     * @throws Exception
     */
    public static final Properties nactiProperties() throws Exception {
        properties = new XProperties();
        properties.load(new FileInputStream("src/main/diplomka.properties"));
        
        setSlozitejsiObvod(properties.getProperty("slozitejsi.obvod").equalsIgnoreCase("true"));

        return properties;
    }

    /**
     * Metoda vrati properties pro cely program.
     *
     * @return properties
     */
    public static final Properties getProperties() {
        return properties;
    }

    /**
     * Metoda nacte data ze vstupniho souboru a vrati je jako List<RadekDat>.
     *
     * @param soubor soubor, z nehoz se nacitaji data    
     * @return dataZeSouboru
     * @throws Exception
     */
    public static List<RadekDatDavka> nactiDataZeSouboru(String soubor) throws Exception {
        List<RadekDatDavka> dataZeSouboru = new ArrayList<RadekDatDavka>();
        FileReader reader = new FileReader(soubor);
        BufferedReader bufferedReader = new BufferedReader(reader);
        String radka;
        String delims = "[ ]+";
        RadekDatDavka radekDat;

        while ((radka = bufferedReader.readLine()) != null) {
            String[] tokens = radka.split(delims);           
            radekDat = new RadekDatDavka(tokens[0], tokens[1], Integer.valueOf(tokens[2]));
            dataZeSouboru.add(radekDat);
        }       
        reader.close();
        return dataZeSouboru;
    }

    /**
     * Metoda nacte data ze vstupniho souboru a vrati je jako List<VektorAFitness>.
     *
     * @param soubor soubor, z nehoz se nacitaji data    
     * @return List<VektorAFitness>
     * @throws FileNotFoundException
     * @throws IOException
     * @throws Exception
     */
    public static List<VektorAFitness> nactiDataZeSouboruJenVektory(String soubor) throws FileNotFoundException, IOException, Exception {
        List<VektorAFitness> dataZeSouboru = new ArrayList<VektorAFitness>();
        FileReader reader = new FileReader(soubor);
        BufferedReader bufferedReader = new BufferedReader(reader);
        String radka;
        VektorAFitness vektorAFitness;

        while ((radka = bufferedReader.readLine()) != null) {
            vektorAFitness = new VektorAFitness(radka.trim(), spoctiFitness(radka.trim()));
            dataZeSouboru.add(vektorAFitness);
        }

        reader.close();
        return dataZeSouboru;
    }

    /**
     * Metoda precte jeden radek dat ze souboru a vrati ho jako String.
     *
     * @param vstupniSoubor
     * @return radek dat ze vstupniho souboru
     * @throws Exception
     */
    public static final String nactiRadekDat(String vstupniSoubor) throws Exception {       
        if (!DipValidators.jeParametrValidni(vstupniSoubor)) {           
            throw new Exception(properties.getProperty("soubor.vstup.error"));
        }

        FileReader fileReader = new FileReader(vstupniSoubor);
        BufferedReader bufferedReader = new BufferedReader(fileReader);
        String data = bufferedReader.readLine();
        bufferedReader.close();      
        return data;
    }

    /**
     * Metoda vrati pocet testovacich vektoru v danem souboru.
     *
     * @param vstupniSoubor
     * @return vrati pocet vektoru v souboru
     * @throws Exception
     */
    public static final int getPocetVektoruVSouboru(String vstupniSoubor) throws Exception {
        if (!DipValidators.jeParametrValidni(vstupniSoubor)) {            
            throw new Exception(properties.getProperty("soubor.vstup.error"));
        }

        FileReader fileReader = new FileReader(vstupniSoubor);
        BufferedReader bufferedReader = new BufferedReader(fileReader);
        int pocetVektoru = 0;
        while (bufferedReader.readLine() != null) {
            pocetVektoru++;
        }
        bufferedReader.close();

        return pocetVektoru;
    }

    /**
     * Metoda provede test ekvivalence dvou vektoru a vrati vysledek porovnani.
     * Bere v potaz i do not care symboly, cili za 'x' muze byt dosazena jak 1, tak 0.
     *
     * @param vektorA
     * @param vektorB
     * @return jsou ekvivalentni
     */
    private static boolean jsouEkvivalentni(String vektorA, String vektorB) {
        if (vektorA.length() != vektorB.length()) {
            return false;
        }

        for (int i = 0; i < vektorA.length(); i++) {
            if (vektorA.charAt(i) != vektorB.charAt(i)) {
                if (vektorA.charAt(i) != 'x' && vektorB.charAt(i) != 'x') {
                    return false;
                }
            }

            if (i == vektorA.length() - 1) {
                return true;
            }
        }
        return false;
    }

    /**
     * Metoda dle zadaneho vzoroveho vektoru najde v souboru vsechny ekvivalentni 
     * vektory a vrati jejich List<RadekDat>.
     *
     * @param vzor
     * @param dataZeSouboru
     * @return list ekvivalentnich vektoru
     * @throws Exception
     */
    public static List<RadekDat> getEkvivalentniVektory(String vzor, List<RadekDat> dataZeSouboru) throws Exception {
        List<RadekDat> ekvivalentniVektory = new ArrayList<RadekDat>();

        for (RadekDat radekDat : dataZeSouboru) {
            if (jsouEkvivalentni(vzor, radekDat.getVektor())) {
                ekvivalentniVektory.add(radekDat);
            }
        }

        if (ekvivalentniVektory.size() == 0) {            
            throw new Exception(properties.getProperty("ekvivalentni.vektory.pocet.error"));
        }

        return ekvivalentniVektory;
    }

    /**
     * Metoda vrati vyslednou chybovou masku, ktera vznikne operaci OR pres vsechny chybove masky
     * vektoru ekvivalentnich se vzorovym vektorem.
     * 
     * @param ekvivalentniVektory
     * @return vyslednou chybovou masku
     * @throws Exception
     */
    public static String getChybovouMasku(List<RadekDat> ekvivalentniVektory) throws Exception {
        String chybovaMaska = ekvivalentniVektory.get(0).getChybovaMaska();

        for (RadekDat radekDat : ekvivalentniVektory) {
            chybovaMaska = getVysledekOperaceOR(chybovaMaska, radekDat.getChybovaMaska());
        }
        return chybovaMaska;
    }

    /**
     * Metoda vykona logickou operaci OR na dvema chybovymi maskami a vrati vysledek teto operace.
     * 
     * @param chybMaskaA
     * @param chybMaskaB
     * @return vysledek operace OR
     * @throws Exception
     */
    public static String getVysledekOperaceOR(String chybMaskaA, String chybMaskaB) throws Exception {
        if (chybMaskaB == null || chybMaskaB.length() == 0 || chybMaskaB.equals("")) {
            return chybMaskaA;
        }

        if (chybMaskaA.length() != chybMaskaB.length()) {
            Thread.sleep(500);
            throw new Exception(properties.getProperty("chybova.maska.delka.error"));
        }

        StringBuffer chybMaskaBuffer = new StringBuffer();
        chybMaskaBuffer.append(chybMaskaA);
        for (int i = 0; i < chybMaskaA.length(); i++) {
            if (chybMaskaA.charAt(i) == 'x' && chybMaskaB.charAt(i) != 'x') {
                chybMaskaBuffer.setCharAt(i, chybMaskaB.charAt(i));
            }

            if (chybMaskaA.charAt(i) == '0' && chybMaskaB.charAt(i) == '1') {
                chybMaskaBuffer.setCharAt(i, chybMaskaB.charAt(i));
            }
        }
        return chybMaskaBuffer.toString();
    }

    /**
     * Metoda vypise vektor algoritmu.
     *
     * @param vektor
     */
    public static void vypisVektorAlgoritmu(String vektor) {
        System.out.println(vektor);
    }

    /**
     * Metoda zapise vektory do souboru.
     *
     * @param soubor    
     * @param object
     * @param pripis pokud nastaveno na true, bude se pripisovat na konec souboru, pri false se vzdy prepise
     * @throws Exception
     */
    synchronized public static void zapisVektoryDoSouboru(String soubor, Object object, boolean pripis)
            throws Exception {

        List<String> sadaVektoru;

        if (!DipValidators.jeParametrValidni(soubor)) {           
            throw new Exception(properties.getProperty("soubor.vystup.error"));
        }

        if (object instanceof ArrayList) {
            sadaVektoru = (List<String>) object;
        } else {
            sadaVektoru = new ArrayList<String>();
            sadaVektoru.add((String) object);
        }

        if (sadaVektoru == null || sadaVektoru.size() == 0) {            
            throw new Exception(properties.getProperty("mnozina.vektoru.error"));
        }

        File file = new File(soubor);
        FileWriter fileWriter = new FileWriter(file, pripis);
        BufferedWriter buffWriter = new BufferedWriter(fileWriter);

        for (String vektor : sadaVektoru) {
            buffWriter.write(vektor);
            buffWriter.newLine();
        }
        buffWriter.close();
        Thread.sleep(50);
    }

    /**
     * Metoda vyprazdni pozadovany soubor.
     *
     * @param soubor
     * @throws IOException 
     */
    public static void vyprazdniSoubor(String soubor) throws IOException {
        //System.out.println("Mazu soubor! " + soubor);
        File file = new File(soubor);
        if (file.exists()) {
            file.delete();
        }
        try {
            Thread.sleep(250);
        } catch (InterruptedException ex) {
            ex.printStackTrace();
        }
        file.createNewFile();
    }

    /**
     * Metoda vykona rotaci retezce(pararm) o jednu pozici doprava a
     * orotovany retezec vrati.
     *
     * @param param
     * @return orotovany retezec o jednu pozici vpravo
     * @throws Exception
     */
    public static final String rotujString(String param) throws Exception {
        if (param == null) {            
            throw new Exception(properties.getProperty("null.parametr.error"));
        }

        if (param.length() == 1 || param.equals("")) {
            return param;
        }

        StringBuffer rotParam = new StringBuffer(param.length());
        rotParam.append(param.charAt(param.length() - 1));
        rotParam.append(param.substring(0, param.length() - 1));

        return rotParam.toString();
    }

    /**
     * Metoda vygeneruje nahodny seed pozadovane delky a ten vrati.
     *
     * @param delka
     * @return seed pozadovane delky
     * @throws Exception
     */
    public static final String generujSeed(int delka) throws Exception {
        if (delka <= 0) {            
            throw new Exception(properties.getProperty("index.error"));
        }

        StringBuffer seed = new StringBuffer();
        for (int i = 0; i < delka; i++) {
            seed.append(new Random().nextInt(2));
        }

        return seed.toString();
    }

    /**
     * Metoda vygeneruje nahodny polynom stupne "stupen" a delky "stupen+1" a ten vrati.
     * x^5+x^3+1 = 101001
     * 
     * @param stupen
     * @return polynom
     * @throws Exception
     */
    public static final String generujPolynom(int stupen) throws Exception {
        if (stupen <= 0) {            
            throw new Exception(properties.getProperty("index.error"));
        }

        StringBuffer polynom = new StringBuffer();
        polynom.append("1");
        for (int i = 1; i < stupen - 1; i++) {
            polynom.append(new Random().nextInt(2));
        }
        polynom.append("1");
        return polynom.toString();
    }

    /**
     * Metoda provede jeden krok LZPR a vrati nove vznikly vektor.
     *
     * @param polynom
     * @param seed
     * @return vrati vektor vznikly krokem LZPR
     * @throws Exception
     */
    public static String krokLZPR(String polynom, String seed) throws Exception {
        if (polynom == null || seed == null || polynom.length() != seed.length()) {
            throw new Exception(properties.getProperty("seed.polynom.validace.error"));
        }

        String novyVektor = seed;

        novyVektor = DipUtils.rotujString(novyVektor);
        if (novyVektor.charAt(0) == '1') {
            novyVektor = xor(polynom, novyVektor);
        }
        return novyVektor;
    }

    /**
     * Metoda provede logickou operaci XOR (v duchu LZPR) dvou binarnich retezcu a
     * vrati vysledek teto operace.
     *
     * @param polynom
     * @param aktStav
     * @return polynom XOR aktStav
     */
    public static String xor(String polynom, String aktStav) {
        StringBuffer aktBuffer = new StringBuffer(aktStav);
        for (int i = 0; i < polynom.length() - 2; i++) {
            if (polynom.charAt(i) == '1' && aktStav.charAt(i + 1) == '1') {
                aktBuffer.replace(i + 1, i + 2, "0");
            } else if (polynom.charAt(i) == '1' && aktStav.charAt(i + 1) == '0') {
                aktBuffer.replace(i + 1, i + 2, "1");
            }
        }
        return aktBuffer.toString();
    }


    /**
     * Metoda spusti externi proces. Obvykle bat soubor s parametry pro Atalantu.
     *
     * @param oznaceniObvodu napr. c499, c1355, ...
     * @param batSoubor *.bat soubor pro spusteni Atalanty
     * @throws Exception
     */
    public static final void spustExternal(String oznaceniObvodu, String batSoubor) throws Exception {      
        if (!DipValidators.jeParametrValidni(oznaceniObvodu)) {           
            throw new Exception(properties.getProperty("soubor.vystup.error"));
        }

        if (!DipValidators.jeParametrValidni(batSoubor)) {            
            throw new Exception(properties.getProperty("soubor.bat.atalanta.error"));
        }
        
        Runtime runtime = Runtime.getRuntime();        
        Process process;      
        synchronized (runtime) {
            if (!isSlozitejsiObvod()) {
                process = runtime.exec(batSoubor + " " + oznaceniObvodu);
            } else {
                //pro slozitejsi obvod pouzij variantu bez parametru oznaceniObvodu
                process = runtime.exec(batSoubor);
            }
        }        
        process.waitFor();
        process.destroy();
       
    }
    
    /**
     * Metoda vrati vysledky generovane Atalantou.
     *
     * @param nastavPocetDetek nastavPocDetekP zda nastavit pocet detekovatelnych poruch
     * @return vysledky generovane Atalantou
     * @throws IOException
     */
    public static List<String> nactiVysledkyAtalanty(boolean nastavPocetDetek) throws IOException {
        //System.out.println("Nacitam vysledky Atalanty.");       
        List<String> vysledky = new ArrayList<String>();
        File file = new File(properties.getProperty("soubor.atalanta.rep"));        

        FileReader reader = reader = new FileReader(file);
        BufferedReader bufferedReader = new BufferedReader(reader);

        String radka = null;
        while ((radka = bufferedReader.readLine()) != null) {
            if (radka.contains(D_FAULTS_STRING)) {
                if (nastavPocDetekP) {                   
                    setAktPocetDetekPoruch(radka);                   
                }
                if (nastavPocetDetek) {                    
                    setPocetDetekPoruch(radka);
                    nastavPocDetekP = true;
                }
            }
            vysledky.add(radka);
        }
        bufferedReader.close();
        
        return vysledky;
    }

    /**
     * Metoda nacte vysledky Atalanty, konkretne pocet detekovanych poruch
     * a zapise je do souboru.
     *        
     * @throws IOException
     */
    public static void nactiVysledkyAtalanty() throws IOException {                
        File file = new File(properties.getProperty("soubor.atalanta.rep"));

        FileReader reader = reader = new FileReader(file);
        BufferedReader bufferedReader = new BufferedReader(reader);

        String radka = null;
        while ((radka = bufferedReader.readLine()) != null) {
            if (radka.contains(D_FAULTS_STRING)) {
                setAktPocetDetekPoruch(radka);
            }            
        }
        bufferedReader.close();       
    }

    /**
     * Metoda zapise do souboru param.
     *
     * @param seedPoly
     * @throws IOException
     */
    public static void zapisSeedPolyDoSouboru(String seedPoly) throws IOException {
        FileWriter writer = new FileWriter(properties.getProperty("soubor.detekovane.poruchy"), true);
        BufferedWriter bufferedWriter = new BufferedWriter(writer);

        bufferedWriter.write(seedPoly);
        bufferedWriter.newLine();
        bufferedWriter.close();
    }

    /**
     * Metoda sekvencne zapisuje pocet detekovanych poruch do souboru.
     *
     * @param pocetDetek
     * @throws IOException
     */
    public static void zapisPocetDetekDoSouboru(int pocetDetek) throws IOException {
        FileWriter writer = new FileWriter(properties.getProperty("soubor.detekovane.poruchy"), true);
        BufferedWriter bufferedWriter = new BufferedWriter(writer);

        bufferedWriter.write(String.valueOf(pocetDetek));
        bufferedWriter.newLine();
        bufferedWriter.close();
    }

    /**
     * Metoda sekvencne zapisuje pocet detekovanych poruch do souboru.
     *
     * @param poruchy
     * @throws Exception
     */
    public static void zapisPocetDetekDoSouboru(List<Integer> poruchy) throws Exception {
        FileWriter writer = new FileWriter(properties.getProperty("soubor.detekovane.poruchy"), true);
        BufferedWriter bufferedWriter = new BufferedWriter(writer);

        for (int i = 0; i < poruchy.size(); i++) {
            bufferedWriter.write(String.valueOf(poruchy.get(i)));
            bufferedWriter.newLine();
        }

        bufferedWriter.close();
    }

    /**
     * Metoda sekvencne zapisuje pocet detekovanych poruch do souboru.
     *
     * @param poruchy
     * @param soubor 
     * @throws Exception
     */
    public static void zapisPocetDetekDoSouboru(List<Integer> poruchy, String soubor) throws Exception {
        FileWriter writer = new FileWriter(soubor, true);
        BufferedWriter bufferedWriter = new BufferedWriter(writer);

        for (int i = 0; i < poruchy.size(); i++) {
            bufferedWriter.write(String.valueOf(poruchy.get(i)));
            bufferedWriter.newLine();
        }

        bufferedWriter.close();
    }

    /**
     * Metoda sekvencne zapisuje pocet detekovanych poruch do souboru.
     *
     * @param pocetDetek
     * @param soubor
     * @throws Exception
     */
    public static void zapisPocetDetekDoSouboru(int pocetDetek, String soubor) throws Exception {
        FileWriter writer = new FileWriter(soubor, true);
        BufferedWriter bufferedWriter = new BufferedWriter(writer);

        bufferedWriter.write(String.valueOf(pocetDetek));
        bufferedWriter.newLine();

        bufferedWriter.close();
    }

    /**
     * Metoda zapise do souboru pocet (priblizne) nutnych vektoru pro
     * detekci vsech detekovatelnych poruch.
     *    
     * @param nutnychVektoru pocet nutnych vektoru
     * @param cas doba behu v sec.
     * @throws IOException
     */
    public static void zapisPocetNutnychVektoru(String nutnychVektoru, String cas) throws IOException {
        FileWriter writer = new FileWriter(properties.getProperty("soubor.nutne.vektory"), true);
        BufferedWriter bufferedWriter = new BufferedWriter(writer);

        bufferedWriter.write(nutnychVektoru + "     " + cas);
        bufferedWriter.newLine();
        bufferedWriter.close();
    }

    /**
     * Metoda vrati maximalni pocet detekovatelnych poruch.
     *
     * @return pocet detekovatelnych poruch
     */
    public static String getMaxPocetDetekovatPoruch() {       
        return pocetDetekovatPoruch;
    }

    /**
     * Metoda nastavi pocet detekovatelnych poruch.
     *
     * @param radka
     */
    private static void setPocetDetekPoruch(String radka) {
        String delims = "[ ]+";

        String[] tokens = radka.split(delims);
        pocetDetekovatPoruch = tokens[1];
        //System.out.println(pocetDetekovatPoruch);
    }

    /**
     * Metoda zjisti a nastavi aktualni pocet detekovanych poruch.
     *
     * @param radka     
     */
    public static void setAktPocetDetekPoruch(String radka) {
        String delims = "[ ]+";

        String[] tokens = radka.split(delims);
        //System.out.println("Setuju akt. pocet detek. poruch na " +  tokens[1]);
        aktPocetDetekPoruch = tokens[1];       
    }

    /**
     * Metoda z chybove masky urci pocet detekovanych poruch.
     *
     * @param chybMaska
     * @return pocet detekovanych poruch
     * @throws Exception
     */
    public static int getPocetDetekPoruchZMasky(String chybMaska) throws Exception {
        int detekChyb = 0;

        if (chybMaska == null || chybMaska.length() == 0 || chybMaska.equals("")) {
            throw new Exception(properties.getProperty("chybova.maska.error"));
        }

        for (int i = 0; i < chybMaska.length(); i++) {
            if (chybMaska.charAt(i) == '1') {
                detekChyb++;
            }
        }

        return detekChyb;
    }

    /**
     * Metoda spocte a vrati pocet poruch, ktere vektor detekuje,
     * cili pocet jednicek v danem vektoru.
     *
     * @param vektor
     * @return pocet detekovanych poruch
     */
    public static int spoctiFitness(String vektor) {
        int detekPoruch = 0;
        for (int i = 0; i < vektor.length(); i++) {
            if (vektor.charAt(i) == '1') {
                detekPoruch++;
            }
        }
        return detekPoruch;
    }   

    /**
     * Metoda vrati aktualni pocet detekovanych poruch.
     *
     * @return akt. pocet detekovanych poruch
     */
    public static int getAktPocetDetekPoruch() {
        return Integer.valueOf(aktPocetDetekPoruch);
    }

    /**
     * Metoda nastavi pocet akt. detekovanych poruch.
     *
     * @param aktPocetDetekPoruch 
     */
    public static void setPocetAktDetekovatPoruch(String aktPocetDetekPoruch) {
        DipUtils.aktPocetDetekPoruch = aktPocetDetekPoruch;
    }

    /**
     * Metoda zaokrouhli cislo tak, ze pokud obsahuje libovolnou
     * desetinnou cislici ruznou od nuly, vrati cele cislo o jedno vyssi, nez
     * je cela cast zaokrouhlovaneho cisla.
     *
     * @param cisloDouble
     * @return zaokrouhlene cislo
     */
    public static int zaokrouhli(double cisloDouble) {
        if ((int) cisloDouble == 0) {
            return 0;
        }
        String cisloString = String.valueOf(cisloDouble).replace('.', ',');
        String[] tokens = cisloString.split(",");
        if (tokens[1].length() >= 3) {
            tokens[1] = tokens[1].substring(0, 3);
        }
        
        if (Integer.valueOf(tokens[1]) != 0) {
            return Integer.valueOf(tokens[0]);
        }

        return Integer.valueOf(tokens[0]) - 1;
    }

   /**
    * Metoda spocte a vrati prcentualni prirustek detekovanych poruch.
    *
    * @param aktDetek
    * @param refDetek
    * @return procentualni prirustek detekovanych poruch
    * @throws Exception
    */
    public static int spoctiPrirustek(int aktDetek, int refDetek) throws Exception {
        return (int)Math.round( (new Double(aktDetek - refDetek)/aktDetek)*100);
    }


    /**
     * Metoda provede logickou operaci xor a vrati vysledek teto operace
     *
     * @param par1
     * @param par2
     * @return vysledek operace xor
     * @throws Exception
     */
    public static int bitXor(char par1, char par2) throws Exception {      
        if ( (par1 != '0' && par1 != '1') || (par2 != '0' && par2 != '1')) {
            throw new Exception(properties.getProperty("char.bitxor.error"));
        }
   
        if (par1 != par2) {
            return 1;
        }
        
        return 0;
    }

    /**
     * Metoda provede logickou operaci or a vrati vysledek teto operace
     *
     * @param par1
     * @param par2
     * @return vysledek operace or
     * @throws Exception
     */
    public static int bitOr(char par1, char par2) throws Exception {
        if ( (par1 != '0' && par1 != '1') || (par2 != '0' && par2 != '1')) {
            throw new Exception(properties.getProperty("char.bitxor.error"));
        }

        if (par1 == '1' || par2 == '1') {
            return 1;
        }
        
        return 0;
    }
    /**
     * Metoda vrati vysledek operace OR nad dvema binarnimi vektory.
     *
     * @param par1
     * @param par2
     * @return vysledek operace OR(par1, par2)
     * @throws Exception
     */
    public static String getVektorOR(String par1, String par2) throws Exception {
        if (par1 == null || par2 == null) {
            throw new Exception(properties.getProperty("null.parametr.error"));
        }

        if (par1.length() != par2.length()) {
            throw new Exception(properties.getProperty("delky.parametr.error"));
        }

        StringBuffer buffer = new StringBuffer(par1.length());
        for (int i = 0; i < par1.length(); i++) {
            buffer.append(bitOr(par1.charAt(i), par2.charAt(i)));
        }

        return buffer.toString();
    }

    /**
     * Metoda vrati novy seed vznikly z reseedingu.
     *
     * @return novy seed
     * @throws Exception
     */
    public static String getNovySeed() throws Exception {
        String novySeed;        
        System.out.println("Get novy seed!");

        if (!isSlozitejsiObvod()) {
            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("lzpr.soubor.bat"));

            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"),
                properties.getProperty("reseeding1.soubor.bat"));
        } else {
            String atalantaKrokLZPR = properties.getProperty("atalanta.lzpr");
            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"), atalantaKrokLZPR);

            String atalantaReseeding = properties.getProperty("atalanta.reseeding");
            DipUtils.spustExternal(properties.getProperty("oznaceni.obvodu"), atalantaReseeding);
        }

        novySeed = DipUtils.nactiRadekDat(properties.getProperty("soubor.nedetekovane.poruchy"));        
        return novySeed;
    }

    /**
     * Metoda na vstupni vektor aplikuje pravidlo 90 a vrati novy vektor dle
     * onoho pravidla.
     *
     * @param vstupniVektor
     * @return vektor po aplikaci pravidla 90
     * @throws Exception
     */
    public static String getVektorPravidlo90(String vstupniVektor) throws Exception {        
        if (vstupniVektor == null || vstupniVektor.equals("") || vstupniVektor.length() == 0) {
            throw new Exception(properties.getProperty("nulovy.vychozi.stav.error"));
        }
        
        StringBuffer buffer = new StringBuffer(vstupniVektor);              
        int[] xors = new int[vstupniVektor.length() - 2];

        for (int i = 0; i < vstupniVektor.length()-2; i++) {
            xors[i] = bitXor(vstupniVektor.charAt(i), vstupniVektor.charAt(i+2));           
        }        

        for (int i = 0; i < xors.length; i++) {
            buffer.replace(i + 1, i + 2, String.valueOf(xors[i]));
        }
        
        return buffer.toString();
    }

     /**
     * Metoda na vstupni vektor aplikuje pravidlo 60 a vrati novy vektor dle
     * onoho pravidla.
     *
     * @param vstupniVektor
     * @return vektor po aplikaci pravidla 60
     * @throws Exception
     */
    public static String getVektorPravidlo60(String vstupniVektor) throws Exception {
        if (vstupniVektor == null || vstupniVektor.equals("") || vstupniVektor.length() == 0) {
            throw new Exception(properties.getProperty("nulovy.vychozi.stav.error"));
        }

        StringBuffer buffer = new StringBuffer();
        buffer.append(bitXor(vstupniVektor.charAt(0), vstupniVektor.charAt(vstupniVektor.length()-1)));

        for (int i = 0; i < vstupniVektor.length() - 1; i++) {
            buffer.append(bitXor(vstupniVektor.charAt(i), vstupniVektor.charAt(i + 1)));
        }

        return buffer.toString();
    }

    /**
     * Metoda nastavi defaultne chybovou masku na same nuly.
     *
     * @param delka delka chybove masky
     * @return defaultni chybova maska
     * @throws Exception
     */
    public static String setDefaultChybMaska(int delka) throws Exception {
        if (delka <= 0 || delka >= 1000) {
            System.out.println(properties.getProperty("chybova.maska.delka.default.error"));
        }

        StringBuffer buffer = new StringBuffer(delka);
        for (int i = 0; i < delka; i++) {
            buffer.append("0");
        }

        return buffer.toString();
    }

    public static boolean isSlozitejsiObvod() {
        return slozitejsiObvod;
    }

    public static void setSlozitejsiObvod(boolean slozitejsiObvod) {
        DipUtils.slozitejsiObvod = slozitejsiObvod;
    }

}

