package utility;
import java.util.Properties;

/**
 * Trida rozsiruje moznosti property souboru. Property muze odkazovat na klic jine
 * propery a ziska tak jeji hodnotu.
 */
public class XProperties extends Properties {
   
    private static final String START_CONST = "{";
    private static final String END_CONST = "}";   
    private static final int MAX_SUBST_DEPTH = 5;

   /**
    * Konstruktor tridy.
    */
    public XProperties() {
        super();
    }

    /**
     * Vytvori prazdny (defaultni) property soubor.
     * @param prop
     */
    public XProperties(Properties prop) {
        super(prop);
    }

    /**
     * Metoda vyhleda property dle zadaneho klice a
     * vrati hodnotu dane property.
     *
     * @param klic
     * @return hodnota property
     */
    @Override
    public String getProperty(String klic) {
        // Return the property value starting at level 0
        return getProperty(klic, 0);
    }

    /**
     * Metoda vyhleda property dle zadaneho klice a
     * vrati hodnotu dane property a rekurzivne nahradi odkaz
     * na vyhledanou property jeji hodnotou
     *
     * @param klic klic k property
     * @param level uroven zanoreni, max 5
     * @return hodnota property
     */
    private String getProperty(String klic, int level) {
        String hodnota = super.getProperty(klic);

        if (hodnota != null) {            
            int startIndex = 0;
            int startJmeno = hodnota.indexOf(START_CONST, startIndex);

            while (startJmeno != -1) {
                if (level + 1 > MAX_SUBST_DEPTH) {                    
                    return hodnota;
                }

                int konecJmeno = hodnota.indexOf(END_CONST, startJmeno);
                if (konecJmeno == -1) {
                    return hodnota;
                }

                String konstJmeno = hodnota.substring(startJmeno + 1, konecJmeno);
                String konstHodnota = getProperty(konstJmeno, level + 1);

                if (konstHodnota == null) {
                    return hodnota;
                }

                //nahradi odkaz na property jeji hodnotou
                String newValue = (startJmeno > 0) ? hodnota.substring(0, startJmeno) : "";
                newValue += konstHodnota;

                startIndex = newValue.length();

                newValue += hodnota.substring(konecJmeno + 1);
                hodnota = newValue;
                startJmeno = hodnota.indexOf(START_CONST, startIndex);
            }
        }      
        return hodnota;
    }
}
